; Lifts 30 lb. PWM starts at 127 on & decays to 5
;Stepinator Steps positive edge triggered on RB0/INT
;On time decays to short pulses and off time increases for each step.  
	LIST	p=16F628A		;tell assembler what chip we are using
	include "P16F628A.inc"		;include the defaults for the chip
;	__config 0x3D18		;sets the configuration settings
	__config 0x3D38		;sets the configuration settings  MCLR on
	ERRORLEVEL	0,	-302	;suppress bank selection messages
	radix dec			;default base 10
	cblock 	0x20 			;start of general purpose registers
		count			;used in TableHigh read routine
		count1 			;used in delay routine
		counta 			;used in delay routine
		countb 			;used in delay routine
		hiside			;Holds indexed high side bits
		loside			;Holds indexed low side bits
		ontime			;of PMW
		offtime			;of PMW
	endc

LEDPORT	Equ	PORTB		;set constant LEDPORT = 'PORTB'
LEDTRIS	Equ	TRISB		;set constant for TRIS register
	
	org	0x0000			;org sets the origin, 0x0000 for the 16F628,
					;this is where the program starts running	
	movlw	0x07
	movwf	CMCON		;turn comparators off (make it like a 16F84)

	clrf	LEDPORT			;set all outputs low before TRIS
   	bsf 	STATUS,		RP0	;select bank 1
   	movlw 	b'10000001'		;set PortB RB0 input and 1 to 6 outputs
   	movwf 	LEDTRIS			;RB7 input for direction
	bsf		OPTION_REG,	INTEDG	;Rising edge trigger
	bcf		STATUS,		RP0	;select bank 0
	clrf	LEDPORT			;set all outputs low
   	bsf 	INTCON,	INTE 	; enable INT pin
   	bcf 	INTCON,	INTF 	; Clear INT flag

Start
	clrf	count			;set counter register to zero

Read				;2 on & 200 0ff is 10Amp slow, almost 0Amp fast
	movlw	2  		;2 on & 100 0ff is 25Amp slow, almost 0Amp fast
	movwf	ontime	
	movlw	100 ;100   	;inc off delay time if less than 200 = 5 to 15 amp stepping
	movwf	offtime	;but still 25Amp slow, almost 0Amp fast
	movf	count, w		;put counter value in W
	call	TableHigh
	movwf	hiside			;Holds indexed high side bits
	movf	count, w		;put counter value in W
	call	TableLow
	movwf	loside			;Holds indexed low side bits

Loop	
	movf	loside,	w			;Holds indexed low side bits
	movwf	PORTB		;set low side bit on

	movf	offtime,w	;for on delay time
	call	Delay	;this waits for a while!

	movf	hiside,	w			;Holds indexed high side bits
	movwf	PORTB	;leaves low side bit on and set high side bit on
	movf	ontime,	w		;for on delay time
	call	Delay			;this waits for a while!

	movf	loside,	w		;Holds indexed low side bits
	movwf	PORTB	;leaves low side bit on and turns high side bit off

;Branch on Less Than You use the carry flag, but the subtract instructions are a bit weird. 
;Sublw 5 does not subtract 5 from W - it subtracts W from 5.
;The subtract instruction will set OR clear the flag depending on the result of the subtract.
;	movf	ontime,	w
;	sublw   5				; Works good. ends up mostly low with short high pulses
;        btfss    STATUS,C
;        decf 	ontime,	f	;dec on delay time if greater than 5

	movf	offtime,	w	;inc off delay time if less than 200 = 5 to 15 amp stepping
	sublw   200				;but still 25Amp slow, almost 0Amp fast
    btfsc    STATUS,C		; Works good. ends up mostly low with short high pulses
    incf 	offtime,	f	;inc off delay time if less than 200


   	btfsc 	INTCON,	INTF 	; Test INT flag
	goto	Left			; If set, next step. If not, keep waiting

	goto	Loop			;go back and do it again

Left
   	bcf 	INTCON,	INTF 	; Clear INT flag
	incf	count,	w
	xorlw	d'6'			;check for last entry
	btfsc	STATUS,	Z
	goto	Zero			;if start from Zero
	incf	count,	f		;else do next
	goto	Read

Zero
	clrf	count			;set counter register to zero
	goto	Read



;Right	; Right and Five were in my notes but never tried.
;   bcf 	INTCON,	INTF 	; Clear INT flag
;	decf	count,	w
;	xorlw	d'255'			;check for last entry
;	btfsc	STATUS,	Z
;	goto	Five			;if start from Five
;	decf	count,	f		;else do next
;	goto	Read
;
;Five
;	movlw	5
;	movwf	count			;set counter register to 5
;	goto	Read




TableHigh	ADDWF   PCL, f		;data TableHigh for bit pattern
							; moved left one. RB0 is step input
        retlw   b'00001100'
        retlw   b'00100100'
        retlw   b'00110000'
        retlw   b'00010010'
        retlw   b'01000010'
        retlw   b'01001000'


TableLow	ADDWF   PCL, f		;data TableLow for bit pattern
        retlw   b'00001000'
        retlw   b'00100000'
        retlw   b'00100000'
        retlw   b'00000010'
        retlw   b'00000010'
        retlw   b'00001000'

Delay	
	movwf	count1
Dly
	decfsz	count1	,f
	goto	Dly
	retlw	0x00


	end



; This was in my notes too but never tested.
; If it is right, then put this where it belongs in the program 
; and test it for direction.
;Step		
;   bcf 	INTCON,	INTF 	; Clear INT flag
;	btfss	PORTB,	7
;	goto	Right
;	goto	Left
